# -*- coding: utf-8 -*-
import logging
from datetime import datetime

from sqlalchemy import Column, String, Integer, DateTime, Boolean
from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

from zebe.module import AD_DB_FILE_PATH

# 定义数据库路径
engine = create_engine('sqlite:///' + AD_DB_FILE_PATH, echo=True)
AdModelBase = declarative_base()


# 广告
class Ad(AdModelBase):
    __tablename__ = 'ad'
    id = Column(Integer, primary_key=True, autoincrement=True, index=True)  # ID
    name = Column(String(40), default="", nullable=False)  # 广告名称
    image = Column(String(300), default="", nullable=False)  # 广告图片
    link = Column(String(300), default="", nullable=False)  # 跳转链接
    is_enable = Column(Boolean, default=False, nullable=False, index=True)  # 是否有效
    click_count = Column(Integer)  # 点击次数
    create_time = Column(DateTime, default=datetime.now(), nullable=False)  # 创建时间

    def __repr__(self):
        return "<Ad(name='%s')>" % (self.name)


# 初始化数据引擎
AdModelBase.metadata.create_all(engine)
AdModelSession = sessionmaker(bind=engine)
logging.debug("初始化数据引擎 -> 成功")



# -*- coding: utf-8 -*-
from zebe.module.ad import Ad, AdModelSession
from zebe.service.base import ModelBaseService


# 广告服务
class AdService(ModelBaseService):

    def __init__(self):
        super().__init__(Ad, AdModelSession)



# -*- coding: utf-8 -*-
from zebe.module.blog import BlogArticle, BlogModelSession
from zebe.service.base import ModelBaseService


# 博客文章服务
class BlogArticleService(ModelBaseService):

    def __init__(self):
        super().__init__(BlogArticle, BlogModelSession)



# -*- coding: utf-8 -*-
from zebe.module.remind import Remind, RemindModelSession
from zebe.service.base import ModelBaseService


# 提醒服务
class RemindService(ModelBaseService):

    def __init__(self):
        super().__init__(Remind, RemindModelSession)

    # 按照模式查询项目
    def find_by_mode(self, mode):
        result = self.session.query(self.entity).filter(self.entity.mode == mode).all()
        self.session.close()
        return result



import os

# 根据操作系统的类型，设置不同的路径前缀
prefix = ''
if os.name == 'posix':  # Mac
    prefix = '/usr/zebe/'
elif os.name == 'linux':  # Linux
    prefix = '/home/data/'
elif os.name == 'nt':  # Windows
    prefix = 'D:/sqlite/'

# 组成完整路径
WECHAT_DB_FILE_PATH = prefix + 'zebe-wechat.sqlite'
PROJECT_DB_FILE_PATH = prefix + 'zebe-project.sqlite'
MEMCARD_DB_FILE_PATH = prefix + 'zebe-memcard.sqlite'
CDRVBA_DB_FILE_PATH = prefix + 'zebe-cdrvba.sqlite'
REMIND_DB_FILE_PATH = prefix + 'zebe-remind.sqlite'
BLOG_DB_FILE_PATH = prefix + 'zebe-blog.sqlite'
DIARY_DB_FILE_PATH = prefix + 'zebe-diary.sqlite'
AD_DB_FILE_PATH = prefix + 'zebe-ad.sqlite'
APP_DB_FILE_PATH = prefix + 'zebe-app.sqlite'




# -*- coding: utf-8 -*-
from datetime import datetime
from time import strftime

from zebe.module.project import Project, Task, ProjectModelSession, AutoDailyTask
from zebe.service.base import ModelBaseService


# 项目服务
class ProjectService(ModelBaseService):

    def __init__(self):
        super().__init__(Project, ProjectModelSession)

    # 按照年份和类型查询项目
    def find_by_year_and_type(self, year, project_type):
        entity = self.entity
        result = self.session.query(entity).filter(entity.year == int(year), entity.type == project_type).all()
        self.session.close()
        return result

    # 按照年份和类型查询项目总数
    def count_by_year_and_type(self, year, project_type):
        entity = self.entity
        result = self.session.query(entity).filter(entity.year == int(year), entity.type == project_type).count()
        self.session.close()
        return result

    # 更新单个任务进度
    def update_progress(self, data_id, progress):
        entity = self.session.query(self.entity).filter(self.entity.id == data_id).one_or_none()
        if entity is not None:
            entity.progress = progress
            self.session.commit()
            self.session.close()

    # 按照名称查找项目
    def find_one_by_name(self, name):
        result = self.session.query(self.entity).filter(self.entity.name == name).one_or_none()
        self.session.close()
        return result


# 任务服务
class TaskService(ModelBaseService):

    def __init__(self):
        super().__init__(Task, ProjectModelSession)

    # 按照年份查询全部任务总数
    def count_all_by_year(self, year):
        entity = self.entity
        result = self.session.query(entity).filter(entity.year == int(year)).count()
        self.session.close()
        return result

    # 按照年份查询已完成任务总数
    def count_finished_by_year(self, year):
        # TODO 1 是个魔法数字，修正为常量
        entity = self.entity
        result = self.session.query(entity).filter(entity.year == int(year), entity.status == 1).count()
        self.session.close()
        return result


# 自动日常任务服务
class AutoDailyTaskService(ModelBaseService):

    def __init__(self):
        super().__init__(AutoDailyTask, ProjectModelSession)

    # 按日期查询全部任务
    def find_all_by_date(self, date):
        entity = self.entity
        result = self.session.query(entity).filter(entity.date == str(date)).all()
        self.session.close()
        return result

    # 按日期删除全部数据
    def delete_all_by_date(self, date):
        self.session.query(self.entity).filter(self.entity.date == str(date)).delete(synchronize_session=False)
        self.session.commit()
        self.session.close()

    # 标记任务完成
    def mark_finish(self, data_id):
        self.__mark_finish_or_unfinish(data_id, True)

    # 标记任务未完成
    def mark_unfinish(self, data_id):
        self.__mark_finish_or_unfinish(data_id, False)

    # 标记任务完成|未完成
    def __mark_finish_or_unfinish(self, data_id, finished):
        entity = self.find_one(data_id)
        if entity is not None:
            entity.status = 1 if finished else 0  # TODO 1和0是魔法数字，改为常量
            self.update(entity)
            # 如果任务归属于某个项目，则自动更新项目的进度
            project_id = entity.project_id
            if project_id is not None:
                project_service = ProjectService()
                project = project_service.find_one(project_id)
                if project is not None:
                    project.finished_task = project.finished_task + 1
                    project.progress = round((project.finished_task / project.total_task) * 100, 2)
                    project_service.update(project)
                    print("任务归属于项目，已自动更新项目的进度")
            else:
                print("任务不归属于任何项目，不需要更新项目的进度")



# -*- coding: utf-8 -*-
import logging
from datetime import datetime

from sqlalchemy import Column, String, Integer, DateTime
from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

from zebe.module import APP_DB_FILE_PATH

# 定义数据库路径
engine = create_engine('sqlite:///' + APP_DB_FILE_PATH, echo=True)
AppModelBase = declarative_base()


# 应用
class App(AppModelBase):
    __tablename__ = 'app'
    id = Column(Integer, primary_key=True, autoincrement=True, index=True)  # ID
    name = Column(String(30), default="", nullable=False, index=True)  # 应用名称
    file_path = Column(String(50), default="", nullable=False)  # 文件路径
    file_name = Column(String(50), default="", nullable=False)  # 文件名称
    log_file = Column(String(50), default="", nullable=False)  # 日志文件
    boot_cmd = Column(String(300), default="", nullable=False)  # 启动命令
    create_time = Column(DateTime, default=datetime.now(), nullable=False)  # 创建时间

    def __repr__(self):
        return "<App(name='%s')>" % (self.name)


# 初始化数据引擎
AppModelBase.metadata.create_all(engine)
AppModelSession = sessionmaker(bind=engine)
logging.debug("初始化数据引擎 -> 成功")



# -*- coding: utf-8 -*-
import logging
from datetime import datetime

from sqlalchemy import Column, String, DateTime, Integer, Text
from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

from zebe.module import BLOG_DB_FILE_PATH

# 定义数据库路径
engine = create_engine('sqlite:///' + BLOG_DB_FILE_PATH, echo=True)
BlogModelBase = declarative_base()


# 博客文章
class BlogArticle(BlogModelBase):
    __tablename__ = 'article'
    id = Column(Integer, primary_key=True, autoincrement=True, index=True)  # ID
    title = Column(String(50), default="", nullable=False)  # 标题
    content = Column(Text, default="", nullable=False)  # 内容
    type = Column(String(10), default="", nullable=False)  # 分类
    tags = Column(String(100), default="", nullable=False)  # 标签
    cover = Column(String(300), default="", nullable=False)  # 封面图
    view_count = Column(Integer, default=0, nullable=False)  # 浏览数
    comment_count = Column(Integer, default=0, nullable=False)  # 评论数
    create_time = Column(DateTime, default=datetime.now(), nullable=False)  # 创建时间

    def __repr__(self):
        return "<BlogArticle(title='%s')>" % (self.title)


# 初始化数据引擎
BlogModelBase.metadata.create_all(engine)
BlogModelSession = sessionmaker(bind=engine)
logging.debug("初始化数据引擎 -> 成功")



import logging
import random
import unittest

from zebe.module.wechat import WeChatMiaoJiUser
from zebe.service.wechat import WeChatMiaoJiUserService


logging.basicConfig(level=logging.DEBUG)


class WechatServiceTest(unittest.TestCase):

    # 测试删除、添加和查询
    def __test_add(self):
        # 删除、查询全部数据
        service = WeChatMiaoJiUserService()
        service.delete_all()
        self.assertTrue(len(service.find_all()) == 0)
        # 添加
        user = WeChatMiaoJiUser()
        user.id = '3'
        user.nickname = 'Zebe'
        user.realname = '张成瑶'
        user.wxid = 'zebe1989'
        service.add(user)
        data_list = service.find_all()
        self.assertTrue(len(data_list) >= 1)
        # 查询单条数据
        data = data_list[0]
        data_find_one = service.find_one(data.id)
        self.assertIsNotNone(data_find_one)
        # 修改
        rand_str = str(random.randint(1, 100000))
        nickname = rand_str + '_rnd_nickname'
        realname = rand_str + '_rnd_realname'
        wxid = rand_str + '_rnd_wxid'
        data_find_one.nickname = nickname
        data_find_one.realname = realname
        data_find_one.wxid = wxid
        service.update(data_find_one)
        # 修改后查询校验是否修改完成
        data_find_one = service.find_one(data.id)
        self.assertTrue(data_find_one.nickname == nickname)
        self.assertTrue(data_find_one.realname == realname)
        self.assertTrue(data_find_one.wxid == wxid)
        # 删除单条数据
        #service.delete_one(data_find_one)
        service.delete_one_by_id(data.id)
        #service.delete_many_by_ids(['1', '2', '3'])
        self.assertTrue(len(service.find_all()) == 0)

    # 测试分页查询
    def test_find(self):
        # 清空数据
        service = WeChatMiaoJiUserService()
        service.delete_all()
        self.assertTrue(len(service.find_all()) == 0)
        # 添加数据1
        user1 = WeChatMiaoJiUser()
        user1.id = '1'
        user1.nickname = 'Zebe1'
        user1.realname = '张成瑶1'
        user1.wxid = 'zebe1989'
        service.add(user1)
        # 添加数据2
        user2 = WeChatMiaoJiUser()
        user2.id = '2'
        user2.nickname = 'Zebe2'
        user2.realname = '张成瑶2'
        user2.wxid = 'zebe1989'
        service.add(user2)
        # 添加数据3
        user3 = WeChatMiaoJiUser()
        user3.id = '3'
        user3.nickname = 'Zebe3'
        user3.realname = '张成瑶3'
        user3.wxid = 'zebe1989'
        service.add(user3)
        # 添加数据4
        user4 = WeChatMiaoJiUser()
        user4.id = '4'
        user4.nickname = 'Zebe4'
        user4.realname = '张成瑶4'
        user4.wxid = 'zebe1989'
        service.add(user4)
        # 查询第一页
        page_data = service.find_by_page(2, 1)
        for data in page_data:
            print(data)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
from datetime import datetime

from sqlalchemy import Column, String, Integer, SmallInteger, Text, Float, DateTime
from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

from zebe.module import PROJECT_DB_FILE_PATH

# 定义数据库路径
engine = create_engine('sqlite:///' + PROJECT_DB_FILE_PATH, echo=True)
ProjectModelBase = declarative_base()


# 项目
class Project(ProjectModelBase):
    __tablename__ = 'project'
    id = Column(Integer, primary_key=True, autoincrement=True, index=True)  # ID
    name = Column(String(40), default="", nullable=False)  # 名称
    type = Column(String(10), default="", nullable=False, index=True)  # 类型
    is_main = Column(SmallInteger, default=0, nullable=False, index=True)  # 是否主要（0=否 1=是）
    description = Column(Text, default="", nullable=False)  # 描述
    year = Column(SmallInteger, default=0, nullable=False, index=True)  # 年份
    relative_url = Column(String(300), default="", nullable=False)  # 相关链接
    cover = Column(String(300), default="", nullable=False)  # 封面图
    total_task = Column(Integer, default=0, nullable=False)  # 总任务数
    finished_task = Column(Integer, default=0, nullable=False)  # 已完成任务数
    estimated_time = Column(Integer, default=-1, nullable=False)  # 预估时长（分）
    progress = Column(Float, default=0.0, nullable=False)  # 进度
    create_time = Column(DateTime, default=datetime.now(), nullable=False)  # 创建时间
    finish_time = Column(DateTime)  # 完成时间

    def __repr__(self):
        return "<Project(name='%s')>" % (self.name)


# 任务
class Task(ProjectModelBase):
    __tablename__ = 'task'
    id = Column(Integer, primary_key=True, autoincrement=True, index=True)  # ID
    title = Column(String(50), default="", nullable=False)  # 标题
    project_id = Column(Integer, default=0, nullable=False, index=True)  # 所属项目ID
    estimated_time = Column(Integer, default=-1, nullable=False)  # 预估时长（分）
    status = Column(SmallInteger, default=0, nullable=False, index=True)  # 状态（0=未完成 1=已完成）
    idx_order = Column(Integer, default=0, nullable=False)  # 排序
    year = Column(SmallInteger, default=0, nullable=False, index=True)  # 年份
    create_time = Column(DateTime, default=datetime.now(), nullable=False)  # 创建时间
    start_time = Column(DateTime)  # 开始时间
    end_time = Column(DateTime)  # 结束时间
    finish_time = Column(DateTime)  # 完成时间

    def __repr__(self):
        return "<Task(title='%s')>" % (self.title)


# 自动日常任务
class AutoDailyTask(ProjectModelBase):
    __tablename__ = 'auto_daily_task'
    id = Column(Integer, primary_key=True, autoincrement=True, index=True)  # ID
    title = Column(String(50), default="", nullable=False)  # 标题
    project_id = Column(Integer)  # 所属项目ID
    estimated_time = Column(Integer, default=-1, nullable=False)  # 预估时长（分）
    status = Column(SmallInteger, default=0, nullable=False, index=True)  # 状态（0=未完成 1=已完成）
    idx_order = Column(Integer, default=0, nullable=False)  # 排序
    year = Column(SmallInteger, default=0, nullable=False, index=True)  # 年份
    date = Column(String(10))  # 日期
    create_time = Column(DateTime, default=datetime.now(), nullable=False)  # 创建时间

    def __repr__(self):
        return "<自动日常任务(title='%s')>" % (self.title)


# 初始化数据引擎
ProjectModelBase.metadata.create_all(engine)
ProjectModelSession = sessionmaker(bind=engine)
logging.debug("初始化数据引擎 -> 成功")



# -*- coding: utf-8 -*-
from zebe.module.cdrvba import CorelDrawVBASubscriber, CorelDrawVBASubscriberModelSession
from zebe.service.base import ModelBaseService


# CorelDRAW VBA订阅用户
class CorelDrawVBASubscriberService(ModelBaseService):

    def __init__(self):
        super().__init__(CorelDrawVBASubscriber, CorelDrawVBASubscriberModelSession)



# -*- coding: utf-8 -*-
import logging

from sqlalchemy import func


class ModelBaseService:
    # 初始化
    def __init__(self, entity, session):
        self.entity = entity
        self.session = session()

    # 校验是否为有效的实体类型
    def __is_valid_entity(self, entity):
        return entity is not None and isinstance(entity, self.entity)

    # 获取实体类的公开属性
    @staticmethod
    def __get_public_fields_of_entity(entity):
        public_fields = []
        if hasattr(entity, '_sa_instance_state'):
            keys = entity._sa_instance_state.attrs._data.keys()
            for field in keys:
                public_fields.append(field)
        logging.info('获取实体类的公开属性成功：' + str(public_fields))
        return public_fields

    # 添加数据
    def add(self, entity):
        if self.__is_valid_entity(entity):
            self.session.add(entity)
            self.session.commit()
            self.session.close()
            logging.info('保存成功')
        else:
            logging.warning('不是自身实体类型，无法添加')

    # 批量添加数据
    def multi_add(self, entity_list):
        if entity_list is not None and isinstance(entity_list, list):
            for entity in entity_list:
                if self.__is_valid_entity(entity):
                    self.session.add(entity)
            self.session.commit()
            self.session.close()
        else:
            logging.warning('不是列表数据类型，无法添加')

    # 修改数据
    def update(self, entity):
        if self.__is_valid_entity(entity):
            session_entity = self.session.query(self.entity).filter(self.entity.id == entity.id).one_or_none()
            if session_entity is not None:
                entity_fields = self.__get_public_fields_of_entity(entity)
                for field in entity_fields:
                    # 过滤掉主键(ID)
                    if field != 'id':
                        if hasattr(session_entity, field):
                            value = getattr(entity, field)
                            setattr(session_entity, field, value)
                            logging.debug('会话中的实体类设置了属性[' + field + ']=' + str(value))
                        else:
                            logging.debug('过滤掉了主键(ID)')
                self.session.commit()
                self.session.close()
                logging.info('修改成功')
            else:
                logging.error('ID为' + str(entity.id) + '的数据不存在，无法修改')
        else:
            logging.warning('不是自身实体类型，无法修改')

    # 查询全部数据条数
    def count_all(self):
        total = self.session.query(self.entity).count()
        self.session.close()
        return total

    # 查询单条数据
    def find_one(self, data_id):
        result = None
        if data_id is not None:
            result = self.session.query(self.entity).filter(self.entity.id == data_id).one_or_none()
            self.session.close()
            logging.info('查询单条数据 -> 成功')
        else:
            logging.warning('查询单条数据 -> 失败，因为ID为空')
        return result

    # 随机查询一条数据
    def find_one_random(self):
        result = self.session.query(self.entity).order_by(func.random()).limit(1).one_or_none()
        self.session.close()
        return result

    # 查询全部数据
    def find_all(self):
        result = self.session.query(self.entity).all()
        self.session.close()
        logging.info('查询全部数据 -> 成功')
        return result

    # 分页查询数据
    def find_by_page(self, page, per_page):
        offset = (page - 1) * per_page
        result = self.session.query(self.entity).order_by(self.entity.id.asc()).offset(offset).limit(per_page).all()
        self.session.close()
        logging.info('分页查询数据 -> 成功')
        return result

    # 根据实例删除单条数据
    def delete_one(self, entity):
        if self.__is_valid_entity(entity):
            self.session.query(self.entity).filter(self.entity.id == entity.id).delete()
            self.session.commit()
            self.session.close()
            logging.info('根据实例删除单条数据 -> 成功')
        else:
            logging.warning('不是自身实体类型，无法删除')

    # 根据ID删除单条数据
    def delete_one_by_id(self, data_id):
        if data_id is not None:
            self.session.query(self.entity).filter(self.entity.id == data_id).delete()
            self.session.commit()
            self.session.close()
            logging.info('根据ID删除单条数据 -> 成功')
        else:
            logging.warning('根据ID删除单条数据 -> 失败，因为ID为空')

    # 根据ID删除多条数据
    def delete_many_by_ids(self, data_ids):
        if data_ids is not None:
            if isinstance(data_ids, list):
                data_list = self.session.query(self.entity).filter(self.entity.id.in_(data_ids))
                for data in data_list:
                    self.session.delete(data)
                self.session.commit()
                self.session.close()
                logging.info('根据ID删除多条数据 -> 成功')
            else:
                logging.warning('根据ID删除多条数据 -> 失败，因为传入的ID参数不是集合')
        else:
            logging.warning('根据ID删除多条数据 -> 失败，因为ID为空')

    # 删除全部数据
    def delete_all(self):
        self.session.query(self.entity).delete(synchronize_session=False)
        self.session.commit()
        self.session.close()
        logging.info('删除全部数据 -> 成功')
